"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
let expectLib;
let setDefaultOptions = () => { };
const extend = () => {
    if (global.expect === undefined) {
        if (!loadExpect()) {
            return;
        }
        global.expect = expectLib;
    }
    else {
        expectLib = global.expect;
    }
    if (!global.expect.extend && !isJasmine()) {
        if (!loadExpect()) {
            return;
        }
        global.ExpectWebdriverIO = expectLib;
        console.warn('Warning! Unsupported expect lib is used.\n' +
            "Only Jasmine >= 3.3.0 and Jest's expect are supported.\n" +
            "expect-webdriverio is assigned to global.ExpectWebdriverIO");
    }
    setDefaultOptions = require('./options').setDefaultOptions;
    const matchers = require('./matchers').default;
    if (expectLib.extend) {
        return expectLib.extend({ ...matchers });
    }
    expectLib = global.jasmine;
    let addMatchersFn = 'addMatchers';
    if (!isWdioSyncInstalled()) {
        addMatchersFn = 'addAsyncMatchers';
    }
    return expectLib.getEnv().beforeAll(function addExpectWebdriverIOMatchers() {
        expectLib[addMatchersFn]({ ...matchers });
    });
};
const isJasmine = () => {
    return global.jasmine && global.expect && global.expectAsync && global.jasmine.getEnv && global.jasmine.addMatchers && global.jasmine.addAsyncMatchers;
};
const loadExpect = () => {
    try {
        return expectLib = require('expect');
    }
    catch (err) {
        return console.error('Failed to load expect package. Make sure it has been installed: npm i expect');
    }
};
const isWdioSyncInstalled = () => {
    try {
        require('@wdio/sync/package.json').version;
        return true;
    }
    catch (_a) {
        return false;
    }
};
extend();
exports.setOptions = setDefaultOptions;
