"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const options_1 = require("./options");
const executeCommand_1 = require("./util/executeCommand");
exports.executeCommand = executeCommand_1.executeCommand;
const elementsUtil_1 = require("./util/elementsUtil");
exports.wrapExpectedWithArray = elementsUtil_1.wrapExpectedWithArray;
exports.updateElementsArray = elementsUtil_1.updateElementsArray;
const formatMessage_1 = require("./util/formatMessage");
exports.enhanceError = formatMessage_1.enhanceError;
exports.numberError = formatMessage_1.numberError;
const config = options_1.getConfig();
const { options: DEFAULT_OPTIONS } = config;
const waitUntil = async (condition, isNot = false, { wait = DEFAULT_OPTIONS.wait, interval = DEFAULT_OPTIONS.interval } = {}) => {
    if (wait === 0) {
        return await condition();
    }
    try {
        let error;
        await browser.waitUntil(async () => {
            error = undefined;
            try {
                return isNot !== await condition();
            }
            catch (err) {
                error = err;
                return false;
            }
        }, wait, undefined, interval);
        if (error) {
            throw error;
        }
        return !isNot;
    }
    catch (err) {
        return isNot;
    }
};
exports.waitUntil = waitUntil;
async function executeCommandBe(received, command, options) {
    const { isNot, expectation, verb = 'be' } = this;
    let el = await received;
    const pass = await waitUntil(async () => {
        const result = await executeCommand_1.executeCommand.call(this, el, async (element) => ({ result: await command(element) }), options);
        el = result.el;
        return result.success;
    }, isNot, options);
    elementsUtil_1.updateElementsArray(pass, received, el);
    const message = formatMessage_1.enhanceErrorBe(el, pass, this, verb, expectation, options);
    return {
        pass,
        message: () => message
    };
}
exports.executeCommandBe = executeCommandBe;
const compareNumbers = (actual, gte, lte, eq) => {
    if (typeof eq === 'number') {
        return actual === eq;
    }
    if (lte > 0 && actual > lte) {
        return false;
    }
    return actual >= gte;
};
exports.compareNumbers = compareNumbers;
exports.compareText = (actual, expected, { ignoreCase = false, trim = false, containing = false }) => {
    if (typeof actual !== 'string') {
        return {
            value: actual,
            result: false
        };
    }
    if (trim) {
        actual = actual.trim();
    }
    if (ignoreCase) {
        actual = actual.toLowerCase();
        expected = expected.toLowerCase();
    }
    if (containing) {
        return {
            value: actual,
            result: actual.includes(expected)
        };
    }
    return {
        value: actual,
        result: actual === expected
    };
};
